#!/usr/bin/env python3

import os
import subprocess
import sys
import tempfile

PROBLEMATIC_CODECS = {'opus', 'aac'}
WORKING_CODECS = {'ac3'}


def get_stream_index(fname):
    """Get the index of the audio stream, validating codec."""
    cmd = ['avprobe', '-show_streams', '-show_format', '-loglevel', 'quiet', fname]
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate()
    out = iter(x.strip() for x in out.decode('utf8').split("\n") if x.strip())
    if err:
        print("ERROR while probing:", err)
        exit()

    index_stored = []

    def parse(lines):
        # convert to a simple dict
        info = dict(line.split("=", 1) for line in lines)
        if info.get('codec_type') != 'audio':
            return

        codec_name = info.get('codec_name')
        if codec_name in WORKING_CODECS:
            print("    ok")
            return

        if codec_name not in PROBLEMATIC_CODECS:
            print("    WARNING: converting from other format?", repr(codec_name))
            return

        index_stored.append(int(info['index']))

    to_parse = []
    in_stream = False
    for line in out:
        if line.startswith("["):
            # delimiter
            if in_stream:
                in_stream = False
                parse(to_parse)
            else:
                if line == "[STREAM]":
                    to_parse = []
                    in_stream = True
            continue
        if in_stream:
            to_parse.append(line)

    if len(index_stored) != 1:
        print("WARNING: no index?", index_stored)
        return
    return index_stored[0]


def main(filenames):
    for fname in filenames:
        print("Probing", repr(fname))
        si = get_stream_index(fname)
        print("    stream index", si)
        if si is None:
            continue

        print("Processing {!r} with format {!r}".format(fname, si))
        _, tmp = tempfile.mkstemp(dir=".", suffix=".mkv")
        os.remove(tmp)
        cmd = ["/usr/bin/avconv", "-loglevel", "warning",
               "-i", fname, "-c", "copy", "-c:{:d}".format(si), "ac3", tmp]
        proc = subprocess.run(cmd)
        if proc.returncode != 0:
            print("ERROR while converting, returncode", proc.returncode)
            exit()

        # ended ok
        os.rename(fname, fname + ".backup")
        os.rename(tmp, fname)


if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Use convert_mkv_audio.py file.mkv [file.mkv [...]]")
        sys.exit()

    main(sys.argv[1:])
